import flowws
from flowws import Argument as Arg
import freud

@flowws.add_stage_arguments
class SmoothBOD(flowws.Stage):
    """Compute and display Bond Orientational Order Diagrams (BOODs)"""
    ARGS = [
        Arg('num_neighbors', '-n', int, default=4,
            help='Number of neighbors to compute'),
        Arg('use_distance', '-d', bool, default=False,
            help='Use distance, rather than num_neighbors, to find bonds'),
        Arg('r_max', type=float, default=2,
            help='Maximum radial distance if use_distance is given'),
    ]

    def run(self, scope, storage):
        """Compute the bonds in the system"""
        box = freud.box.Box.from_box(scope['box'])
        positions = scope['position']

        aq = freud.AABBQuery(box, positions)
        args = dict(num_neighbors=self.arguments['num_neighbors'],
                    exclude_ii=True, r_guess=self.arguments['r_max'])
        if self.arguments['use_distance']:
            args['mode'] = 'ball'
            args['r_max'] = self.arguments['r_max']

        nlist = aq.query(positions, args).toNeighborList()
        rijs = positions[nlist.point_indices] - positions[nlist.query_point_indices]
        self.bonds = box.wrap(rijs)

        scope.setdefault('visuals', []).append(self)

    def draw_plato(self):
        import plato, plato.draw as draw
        prim = draw.SpherePoints(points=self.bonds, on_surface=True)
        scene = draw.Scene(prim, size=(3, 3), pixel_scale=100,
                           features=dict(additive_rendering=dict(invert=True)))
        return scene



import flowws
from flowws import Argument as Arg
import freud

@flowws.add_stage_arguments
class RDF(flowws.Stage):
    """Compute and plot the radial distribution function (RDF)"""
    ARGS = [
        Arg('bins', '-b', int, default=64,
            help='Number of bins to use'),
        Arg('r_min', type=float, default=0,
            help='Minimum radial distance'),
        Arg('r_max', type=float, required=True,
            help='Maximum radial distance'),
    ]

    def run(self, scope, storage):
        """Compute and provide the RDF"""
        compute = freud.density.RDF(
            self.arguments['bins'], self.arguments['r_max'],
            self.arguments.get('r_min', 0))

        box = freud.box.Box.from_box(scope['box'])
        compute.compute((box, scope['position']))
        self.r, self.rdf = compute.bin_centers, compute.rdf

        scope.setdefault('visuals', []).append(self)

    def draw_matplotlib(self, figure):
        ax = figure.add_subplot()
        ax.plot(self.r, self.rdf)
        ax.set_xlabel('r')
        ax.set_ylabel('RDF')
